from cognite.client import CogniteClient
from pandas.testing import assert_frame_equal

from akerbp.mlpet import feature_engineering
from akerbp.mlpet.tests.data.data import (
    DEPTH_TREND_X,
    DEPTH_TREND_Y,
    FORMATION_DF,
    FORMATION_TOPS_MAPPER,
    TEST_DF,
    VERTICAL_DEPTHS_MAPPER,
    VERTICAL_DF,
)

# Assuming COGNITE_API_KEY is set in the environment
CLIENT = CogniteClient(client_name="test", project="akbp-subsurface")


def test_add_formations_and_groups_using_mapper():
    df_with_tops = feature_engineering.add_formations_and_groups(
        FORMATION_DF[["DEPTH", "well_name"]],
        formation_tops_mapper=FORMATION_TOPS_MAPPER,
        id_column="well_name",
    )
    # Sorting columns because column order is not so important
    assert_frame_equal(df_with_tops.sort_index(axis=1), FORMATION_DF.sort_index(axis=1))


def test_add_formations_and_groups_using_client():
    df_with_tops = feature_engineering.add_formations_and_groups(
        FORMATION_DF[["DEPTH", "well_name"]],
        id_column="well_name",
        client=CLIENT,
    )
    assert_frame_equal(df_with_tops.sort_index(axis=1), FORMATION_DF.sort_index(axis=1))


def test_add_formations_and_groups_when_no_client_nor_mapping_is_provided():
    _ = feature_engineering.add_formations_and_groups(
        FORMATION_DF[["DEPTH", "well_name"]],
        id_column="well_name",
    )


def test_add_vertical_depths_using_mapper():
    df_with_vertical_depths = feature_engineering.add_vertical_depths(
        VERTICAL_DF[["DEPTH", "well_name"]],
        vertical_depths_mapper=VERTICAL_DEPTHS_MAPPER,
        id_column="well_name",
        md_column="DEPTH",
    )

    assert_frame_equal(
        df_with_vertical_depths.sort_index(axis=1), VERTICAL_DF.sort_index(axis=1)
    )


def test_add_vertical_depths_using_client():
    df_with_vertical_depths = feature_engineering.add_vertical_depths(
        VERTICAL_DF[["DEPTH", "well_name"]],
        id_column="well_name",
        md_column="DEPTH",
        client=CLIENT,
    )

    assert_frame_equal(
        df_with_vertical_depths.sort_index(axis=1), VERTICAL_DF.sort_index(axis=1)
    )


def test_add_vertical_depths_when_no_client_nor_mapping_is_provided():
    _ = feature_engineering.add_vertical_depths(
        VERTICAL_DF[["DEPTH", "well_name"]],
        id_column="well_name",
        md_column="DEPTH",
    )


def test_add_well_metadata():
    metadata = {"30/11-6 S": {"FOO": 0}, "25/7-4 S": {"FOO": 1}}
    df = feature_engineering.add_well_metadata(
        TEST_DF, metadata_dict=metadata, metadata_columns=["FOO"], id_column="well_name"
    )
    assert "FOO" in df.columns.tolist()


def test_add_depth_trend():
    result = feature_engineering.add_depth_trend(
        DEPTH_TREND_X,
        id_column="well",
        env="test",
        return_file=False,
        return_CI=True,
    )

    assert DEPTH_TREND_Y.equals(result[DEPTH_TREND_Y.columns])



import numpy as np
import pandas as pd

from akerbp.mlpet import feature_engineering, petrophysical_features
from akerbp.mlpet.dataloader import DataLoader
from akerbp.mlpet.tests.test_feature_engineering import CLIENT as client


def test_guess_bs_from_cali():
    input = pd.DataFrame({"CALI": [6.1, 5.9, 12.0, 12.02]})
    df = petrophysical_features.guess_BS_from_CALI(input)
    assert "BS" in df.columns.tolist()


def test_calculate_cali_bs():
    input = pd.DataFrame({"CALI": np.array([6.1, 5.9, 12.0, 12.02])})
    df = petrophysical_features.calculate_CALI_BS(input)
    assert "CALI-BS" in df.columns.tolist()


def test_calculate_VSH():
    dl = DataLoader()
    df = dl.load_from_cdf(
        client=client, metadata={"wellbore_name": "15/3-5", "subtype": "BEST"}
    )
    df["well_name"] = "15/3-5"
    df = petrophysical_features.calculate_LFI(df)
    df = feature_engineering.add_formations_and_groups(
        df, client=client, id_column="well_name", depth_column="DEPTH"
    )
    df_out = petrophysical_features.calculate_VSH(
        df,
        VSH_curves=["GR", "LFI"],
        groups_column_name="GROUP",
        formations_column_name="FORMATION",
        id_column="well_name",
        env="test",
        return_CI=True,
        calculate_denneu=True,
        return_only_vsh_aut=False,
    )
    assert "VSH_AUT" in df_out.columns.tolist()



base_mappings = {
    "curve_mappings": {
        "4A_PRES": "4A_PRESS",
        "A16L": "RSHA",
        "A28H": "RMED",
        "A40L": "RDEP",
        "AC": "AC",
        "ACS": "ACS",
        "AI": "AI",
        "AI10": "RSHA",
        "AI60": "RMED",
        "AI90": "RDEP",
        "AO10": "RSHA",
        "AO60": "RMED",
        "AO90": "RDEP",
        "APLC": "APLC",
        "AT10": "RSHA",
        "AT60": "RMED",
        "AT90": "RDEP",
        "BFV": "BFV",
        "BFV_UHR": "BFV",
        "BPHI": "BPHI",
        "BS": "BS",
        "CAL": "CALI",
        "CALI": "CALI",
        "CALS": "CALI",
        "CGR": "CGR",
        "CMRT": "MRP",
        "CNC": "NEU",
        "DCAV": "CALI",
        "DCOR": "DENC",
        "DEN": "DEN",
        "DENC": "DENC",
        "DEN_VIRGIN": "DEN_VIRGIN",
        "DRHO": "DENC",
        "DT": "AC",
        "DTC": "AC",
        "DTCO": "AC",
        "DTS": "ACS",
        "DTSM": "ACS",
        "FFV": "FFV",
        "FFV_UHR": "FFV",
        "FPRESS": "FPRESS",
        "GR": "GR",
        "GRMA": "GR",
        "GR_EDTC": "GR",
        "HAC": "AC",
        "HCAL": "CALI",
        "HCGR": "CGR",
        "HCNC": "NEU",
        "HCNL": "NEU",
        "HCOR": "DENC",
        "HCORR": "DENC",
        "HDAR": "CALI",
        "HDEN": "DEN",
        "HDRA": "DENC",
        "HDRH": "DENC",
        "HDT": "AC",
        "HFK": "K",
        "HGR": "GR",
        "HPHI": "NEU",
        "HRD": "RDEP",
        "HRHO": "DEN",
        "HRM": "RMED",
        "HRS": "RMIC",
        "HSGR": "SGR",
        "HTHO": "TH",
        "HURA": "U",
        "IDDR": "DENC",
        "IDPE": "PEF",
        "IDRO": "DEN",
        "ILD": "RDEP",
        "ILM": "RMED",
        "K": "K",
        "LLD": "RDEP",
        "LLS": "RSHA",
        "M": "M",
        "MRP": "MRP",
        "MSFL": "RMIC",
        "NEU": "NEU",
        "NPHI": "NEU",
        "NPOR": "NEU",
        "P16L": "RSHA",
        "P28L": "RMED",
        "P40H": "RDEP",
        "P40L": "RDEP",
        "PE": "PEF",
        "PEB": "PEF",
        "PEF": "PEF",
        "PEFZ": "PEF",
        "POTA": "K",
        "RD": "RDEP",
        "RDEP": "RDEP",
        "RH72_1DF": "RH",
        "RHGA": "RHGE",
        "RHGE": "RHGE",
        "RHO8": "DEN",
        "RHOB": "DEN",
        "RHOZ": "DEN",
        "RMED": "RMED",
        "RMIC": "RMIC",
        "RMLL": "RMIC",
        "ROBB": "DEN",
        "ROP": "ROP",
        "RS": "RSHA",
        "RSHA": "RSHA",
        "RT": "RT",
        "RT_HRLT": "RDEP",
        "RV72_1DF": "RV",
        "RXO": "RXO",
        "RXOZ": "RXO",
        "RXO_ADT": "RXO",
        "RXO_HRLT": "RXO",
        "SFLU": "SFLU",
        "SGR": "SGR",
        "SHAR": "RSHA",
        "SP": "SP",
        "SPBR": "SP",
        "TEMP": "TEMP",
        "TH": "TH",
        "THOR": "TH",
        "TNPH": "NEU",
        "U": "U",
        "UCAV": "CALI",
        "URAN": "U",
        "WANH": "WANH",
        "WCAR": "WCAR",
        "WCLA": "WCLA",
        "WCLC": "WCLC",
        "WCOA": "WCOA",
        "WDOL": "WDOL",
        "WEVA": "WEVA",
        "WPYR": "WPYR",
        "WQFM": "WQFM",
        "WSID": "WSID",
        "ZCOR": "DENC",
        "ZDEN": "DEN",
        "ZONE": "lsuName",
        "lsuName": "lsuName",
        "wellName": "well_name",
        "well_name": "well_name",
        "WELL": "well_name",
        "well": "well_name",
    },
    "formations_map": {
        "AGAT": 0,
        "ALKE": 1,
        "AMUNDSEN": 2,
        "ANDREW": 3,
        "ARE": 136,
        "ASGARD": 137,
        "BALDER": 4,
        "BLAREROT": 6,
        "BLODOKS": 5,
        "BORGLUM": 11,
        "BROOM": 7,
        "BRYGGE": 8,
        "BRYNE": 9,
        "BURTON": 10,
        "COOK": 12,
        "DELFIN": 13,
        "DRAKE": 14,
        "DRAUPNE": 15,
        "EGERSUND": 16,
        "EGGA": 17,
        "EIRIKSSON": 18,
        "EKOFISK": 19,
        "ELDFISK": 20,
        "ETIVE": 21,
        "FALK": 22,
        "FARSUND": 23,
        "FENSFJORD": 24,
        "FISKEBANK": 25,
        "FJERRITSLEV": 26,
        "FLEKKEFJORD": 27,
        "FORTIES": 28,
        "FRIGG": 29,
        "FRUHOLMEN": 30,
        "FUGLEN": 31,
        "GARN": 32,
        "GASSUM": 33,
        "GRASEL": 35,
        "GRID": 34,
        "HARDRADE": 36,
        "HAUGESUND": 37,
        "HAVERT": 38,
        "HEATHER": 39,
        "HEIMDAL": 40,
        "HEKKINGEN": 41,
        "HERMOD": 42,
        "HIDRA": 43,
        "HOD": 44,
        "HUGIN": 45,
        "ILE": 46,
        "INTRA BALDER": 47,
        "INTRA DRAUPNE": 48,
        "INTRA HEATHER": 49,
        "INTRA MELKE": 50,
        "INTRA aaSGARD": 51,
        "ISBJORN": 52,
        "JOHANSEN": 53,
        "JORSALFARE": 54,
        "KAI": 55,
        "KLAPPMYSS": 56,
        "KLIPPFISK": 57,
        "KNURR": 58,
        "KOBBE": 59,
        "KOLJE": 60,
        "KOLMULE": 61,
        "KROSSFJORD": 62,
        "KUPFERSCHIEFER": 63,
        "KVEITE": 64,
        "KVITING": 65,
        "KVITNOS": 66,
        "KYRRE": 67,
        "LANGE": 68,
        "LISTA": 69,
        "LOMVI": 70,
        "LUNDE": 71,
        "LYR": 72,
        "LYSING": 73,
        "MANDAL": 74,
        "MAUREEN": 75,
        "MELKE": 76,
        "MIME": 77,
        "MOLO": 78,
        "NANSEN": 79,
        "NAUST": 80,
        "NESS": 81,
        "NISE": 82,
        "NO FORMAL": 83,
        "NORDMELA": 84,
        "NOT": 85,
        "ORN": 134,
        "ORRET": 135,
        "OSEBERG": 86,
        "POLARREV": 87,
        "RAN": 88,
        "RANNOCH": 89,
        "RATTRAY": 90,
        "RAUDE": 91,
        "RODBY": 94,
        "ROGN": 92,
        "ROR": 93,
        "ROYE": 95,
        "SANDNES": 96,
        "SAUDA": 97,
        "SELE": 98,
        "SKADE": 99,
        "SKAGERRAK": 100,
        "SLEIPNER": 101,
        "SMITH": 102,
        "SNADD": 103,
        "SOGNEFJORD": 104,
        "SOLA": 105,
        "SOLDOGG": 106,
        "SPEKK": 107,
        "SPRINGAR": 108,
        "STEINKOBBE": 109,
        "STO": 110,
        "SVARTE": 111,
        "TANG": 112,
        "TARBERT": 113,
        "TARE": 114,
        "TAU": 115,
        "TEIST": 116,
        "TETTEGRAS": 117,
        "TILJE": 118,
        "TOFTE": 119,
        "TOR": 120,
        "TORSK": 121,
        "TRYGGVASON": 122,
        "TUBAN": 123,
        "TUXEN": 124,
        "TY": 125,
        "UGLE": 126,
        "ULA": 127,
        "ULV": 128,
        "UNDIFFERENTIATED": 129,
        "UTSIRA": 130,
        "VADE": 131,
        "VALE": 133,
        "VIDAR": 132,
    },
    "groups_map": {
        "ADVENTDALEN": 0,
        "BASEMENT": 1,
        "BAT": 6,
        "BILLEFJORDEN": 2,
        "BJARMELAND": 3,
        "BOKNFJORD": 4,
        "BRENT": 5,
        "CROMER": 7,
        "DUNLIN": 8,
        "FANGST": 9,
        "FLADEN": 10,
        "GIPSDALEN": 11,
        "GREY": 12,
        "HEGRE": 13,
        "HORDALAND": 14,
        "KAPP": 15,
        "NO DATA": 16,
        "NO GROUP": 17,
        "NORDLAND": 18,
        "NYGRUNNEN": 19,
        "RED": 20,
        "ROGALAND": 21,
        "ROTLIEGEND": 22,
        "SASSENDALEN": 23,
        "SHETLAND": 24,
        "SOTBAKKEN": 25,
        "STATFJORD": 26,
        "TEMPELFJORDEN": 27,
        "TYNE": 28,
        "UNDEFINED": 29,
        "VESTLAND": 30,
        "VIKING": 31,
        "ZECHSTEIN": 32,
    },
}
